@@ -1,9 +1,9 @@
 ;;; tex-info.el --- Support for editing Texinfo source.
 
 ;; Copyright (C) 1993, 1994, 1997, 2000, 2001,
-;;               2004 Free Software Foundation, Inc.
+;;               2004, 2005 Free Software Foundation, Inc.
 
-;; Maintainer: auc-tex@sunsite.dk
+;; Maintainer: auctex-devel@gnu.org
 ;; Keywords: tex
 
 ;; This file is part of AUCTeX.
@@ -26,9 +26,8 @@
 ;;; Code:
 
 (require 'tex)
-(condition-case nil			;Lucid is not providing.
-    (require 'texinfo)
-  (error))
+
+(require 'texinfo)
 
 ;;; Environments:
 
@@ -56,11 +55,20 @@
   "Regexp for environment-like Texinfo list commands.
 Subexpression 1 is what goes into the corresponding `@end' statement.")
 
-(defun Texinfo-insert-environment (env)
-  "Insert Texinfo environment ENV.
-When called interactively, prompt for an environment."
+(defun Texinfo-environment (env &optional arg)
+  "Make Texinfo environment ENV.
+With optional ARG, modify current environment."
+  ;; XXX: This could be enhanced to act like `LaTeX-environment',
+  ;; i.e. suggest a default environment and have its own history.
   (interactive (list (completing-read "Environment: "
-				      Texinfo-environment-list)))
+				      Texinfo-environment-list)
+		     current-prefix-arg))
+  (if arg
+      (Texinfo-modify-environment env)
+    (Texinfo-insert-environment env)))
+
+(defun Texinfo-insert-environment (env)
+  "Insert Texinfo environment ENV."
   (if (and (TeX-active-mark)
 	   (not (eq (mark) (point))))
       (progn
@@ -80,6 +88,19 @@ When called interactively, prompt for an environment."
     (if (null (cdr-safe (assoc "defcv" Texinfo-environment-list)))
 	(forward-line -2))))
 
+(defun Texinfo-modify-environment (env)
+  "Change current environment to environment ENV."
+  (save-excursion
+    (Texinfo-find-env-end)
+    (re-search-backward (concat (regexp-quote TeX-esc) "end \\([a-zA-Z]*\\)")
+			(line-beginning-position))
+    (replace-match env t t nil 1)
+    (beginning-of-line)
+    (Texinfo-find-env-start)
+    (re-search-forward (concat (regexp-quote TeX-esc) "\\([a-zA-Z]*\\)")
+		       (line-end-position))
+    (replace-match env t t nil 1)))
+
 (defun Texinfo-find-env-end ()
   "Move point to the end of the current environment."
   (interactive)
@@ -196,7 +217,7 @@ for @node."
     (define-key map "\C-c\C-u\C-a"   'texinfo-all-menus-update)
 
     ;; Simulating LaTeX-mode
-    (define-key map "\C-c\C-e" 'Texinfo-insert-environment)
+    (define-key map "\C-c\C-e" 'Texinfo-environment)
     (define-key map "\C-c\n"   'texinfo-insert-@item)
     (or (key-binding "\e\r")
 	(define-key map "\e\r" 'texinfo-insert-@item)) ;*** Alias
@@ -287,9 +308,12 @@ for @node."
   
 ;;; Mode:
 
-;;; Do not ;;;###autoload because of conflict with standard texinfo.el.
-(TeX-defun texinfo-mode ()
-  "Major mode in %s for editing Texinfo files.
+;;;###autoload
+(defalias 'Texinfo-mode 'texinfo-mode)
+
+;;;###autoload
+(defun TeX-texinfo-mode ()
+  "Major mode in AUCTeX for editing Texinfo files.
 
 Special commands:
 \\{Texinfo-mode-map}
@@ -387,122 +411,129 @@ value of `Texinfo-mode-hook'."
 				 (TeX-master-file nil nil t))) nil t)
 
   (TeX-add-symbols
-   '("appendix" "Title")
-   '("appendixsec" "Title")
-   '("appendixsection" "Title")
-   '("appendixsubsec" "Title")
-   '("appendixsubsubsec" "Title")
+   '("appendix" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("appendixsec" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("appendixsection" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("appendixsubsec" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("appendixsubsubsec" (TeX-arg-literal " ") (TeX-arg-free "Title"))
    '("asis")
-   '("author" "Author")
+   '("author" (TeX-arg-literal " ") (TeX-arg-free "Author"))
    '("b" "Text")
    '("bullet")
    '("bye")
-   '("c" "Comment")
-   '("center" "Line-of-text")
-   '("chapheading" "Title")
-   '("chapter" "Title")
-;; Texinfo doesn't like the brackets.
-;;   '("cindex" "Entry")
+   '("c" (TeX-arg-literal " ") (TeX-arg-free "Comment"))
+   '("center" (TeX-arg-literal " ") (TeX-arg-free "Line of text"))
+   '("chapheading" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("chapter" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("cindex" (TeX-arg-literal " ") (TeX-arg-free "Entry"))
    '("cite" "Reference")
-   '("clear" "Flag")
-   '("code" "Sample-code")
+   '("clear" (TeX-arg-literal " ") (TeX-arg-free "Flag"))
+   '("code" "Sample code")
    '("command" "Command")
-   '("comment" "Comment")
+   '("comment" (TeX-arg-literal " ") (TeX-arg-free "Comment"))
    '("contents")
-   '("copyright")
-   '("defcodeindex" "Index-name")
-   '("defindex" "Index-name")
+   '("copyright" nil)
+   '("defcodeindex" (TeX-arg-literal " ") (TeX-arg-free "Index name"))
+   '("defindex" (TeX-arg-literal " ") (TeX-arg-free "Index name"))
    '("dfn" "Term")
    '("dmn" "Dimension")
-   '("dots")
+   '("dots" nil)
    '("emph" "Text")
    '("email" "Email address")
-   '("equiv")
+   '("equiv" nil)
    '("error")
    '("evenfooting" Texinfo-lrc-argument-hook)
    '("evenheading" Texinfo-lrc-argument-hook)
    '("everyfooting" Texinfo-lrc-argument-hook)
    '("everyheading" Texinfo-lrc-argument-hook)
-   '("exdent" "Line-of-text")
-   '("expansion")
+   '("exdent" (TeX-arg-literal " ") (TeX-arg-free "Line of text"))
+   '("expansion" nil)
    '("file" "Filename")
    '("finalout")
-   '("findex" "Entry")
-   '("footnote" "Text-of-footnote")
-   '("footnotestyle" "Style")
+   '("findex" (TeX-arg-literal " ") (TeX-arg-free "Entry"))
+   '("footnote" "Text of footnote")
+   '("footnotestyle" (TeX-arg-literal " ") (TeX-arg-free "Style"))
    '("group")
-   '("heading" "Title")
-   '("headings" "On-off-single-double")
+   '("heading" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   ;; XXX: Would be nice with completion.
+   '("headings" (TeX-arg-literal " ") (TeX-arg-free "On off single double"))
    '("i" "Text")
    '("ignore")
-   '("include" "Filename")
-   '("inforef" "Node-name" "Info-file-name")
+   '("include" (TeX-arg-literal " ") (TeX-arg-free "Filename"))
+   '("inforef" "Node name" "Info file name")
    '("item")
    '("itemx")
-   '("kbd" "Keyboard-characters")
-   '("key" "Key-name")
-   '("kindex" "Entry")
-   '("majorheading"  "Title")
+   '("kbd" "Keyboard characters")
+   '("key" "Key name")
+   '("kindex" (TeX-arg-literal " ") (TeX-arg-free "Entry"))
+   '("majorheading" (TeX-arg-literal " ") (TeX-arg-free "Title"))
    '("menu")
    '("minus")
    '("need" "N")
-   '("node" "Name" "Next" "Previous" "Up")
+   '("node" (TeX-arg-literal " ") (TeX-arg-free "Name")
+     (TeX-arg-literal ", ") (TeX-arg-free "Next")
+     (TeX-arg-literal ", ") (TeX-arg-free "Previous")
+     (TeX-arg-literal ", ") (TeX-arg-free "Up"))
    '("noindent")
    '("oddfooting" Texinfo-lrc-argument-hook)
    '("oddheading" Texinfo-lrc-argument-hook)
    '("page")
-   '("paragraphindent" "Indent")
+   '("paragraphindent" (TeX-arg-literal " ") (TeX-arg-free "Indent"))
    '("pindex" "Entry")
-   '("point")
+   '("point" nil)
    '("print")
-   '("printindex" "Index-name")
-   '("pxref" "Node-name")
+   '("printindex" (TeX-arg-literal " ") (TeX-arg-free "Index name"))
+   '("pxref" "Node name")
    '("r" "Text")
-   '("ref" "Node-name")
+   '("ref" "Node name")
    '("refill")
    '("result")
    '("samp" "Text")
    '("sc" "Text")
-   '("section" "Title")
-   '("set" "Flag")
-   '("setchapternewpage" "On-off-odd")
-   '("setfilename" "Info-file-name")
-   '("settitle" "Title")
+   '("section" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("set" (TeX-arg-literal " ") (TeX-arg-free "Flag"))
+   ;; XXX: Would be nice with completion.
+   '("setchapternewpage" (TeX-arg-literal " ") (TeX-arg-free "On off odd"))
+   '("setfilename" (TeX-arg-literal " ") (TeX-arg-free "Info file name"))
+   '("settitle" (TeX-arg-literal " ") (TeX-arg-free "Title"))
    '("shortcontents")
    '("smallbook")
    '("sp" "N")
    '("strong" "Text")
-   '("subheading" "Title")
-   '("subsection" "Title")
-   '("subsubheading" "Title")
-   '("subsubsection" "Title")
-   '("subtitle" "Title")
+   '("subheading" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("subsection" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("subsubheading" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("subsubsection" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("subtitle" (TeX-arg-literal " ") (TeX-arg-free "Title"))
    '("summarycontents")
-   '("syncodeindex" "From-index" "Into-index")
-   '("synindex" "From-index" "Into-index")
+   '("syncodeindex" (TeX-arg-literal " ") (TeX-arg-free "From index")
+     (TeX-arg-literal " ") (TeX-arg-free "Into index"))
+   '("synindex" (TeX-arg-literal " ") (TeX-arg-free "From index")
+     (TeX-arg-literal " ") (TeX-arg-free "Into index"))
    '("t" "Text")
-   '("TeX")
+   '("TeX" nil)
    '("thischapter")
    '("thischaptername")
    '("thisfile")
    '("thispage")
-   '("tindex" "Entry")
-   '("title" "Title")
+   '("tindex" (TeX-arg-literal " ") (TeX-arg-free "Entry"))
+   '("title" (TeX-arg-literal " ") (TeX-arg-free "Title"))
    '("titlefont" "Text")
    '("titlepage")
-   '("today")
-   '("top" "Title")
-   '("unnumbered" "Title")
-   '("unnumberedsec" "Title")
-   '("unnumberedsubsec" "Title")
-   '("unnumberedsubsubsec" "Title")
+   '("today" nil)
+   '("top" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("unnumbered" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("unnumberedsec" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("unnumberedsubsec" (TeX-arg-literal " ") (TeX-arg-free "Title"))
+   '("unnumberedsubsubsec" (TeX-arg-literal " ") (TeX-arg-free "Title"))
    '("value" "Flag")
-   '("var" "Metasyntactic-variable")
-   '("vindex" "Entry")
-   '("vskip" "Amount")
-   '("w" "Text"))
+   '("var" "Metasyntactic variable")
+   '("vindex" (TeX-arg-literal " ") (TeX-arg-free "Entry"))
+   '("vskip" (TeX-arg-literal " ") (TeX-arg-free "Amount"))
+   '("w" "Text")
+   '("xref" "Node name"))
   
-  (run-hooks 'text-mode-hook 'Texinfo-mode-hook)
+  (TeX-run-mode-hooks 'text-mode-hook 'Texinfo-mode-hook)
   (TeX-set-mode-name))
   
 (provide 'tex-info)
