@@ -1,9 +1,9 @@
 ;;; nicefrac.el --- AUCTeX style for the LaTeX package `nicefrac.sty' (v0.9b)
 
-;; Copyright (C) 2004 Free Software Foundation, Inc.
+;; Copyright (C) 2004, 2005 Free Software Foundation, Inc.
 
 ;; Author: Christian Schlauer <cschl@arcor.de>
-;; Maintainer: auc-tex@sunsite.dk
+;; Maintainer: auctex-devel@gnu.org
 ;; Keywords: tex
 
 ;; This file is part of AUCTeX.
@@ -31,12 +31,16 @@
 
 (TeX-add-style-hook
  "nicefrac"
- (function
-  (lambda ()
-    (TeX-add-symbols
-     '("nicefrac" [ "Font changing command" ] "Numerator" "Denominator"))
-    ;; enable fontifying
-    (add-to-list 'font-latex-match-textual-keywords-local "nicefrac")
-    (font-latex-match-textual-make))))
+ (lambda ()
+   (TeX-add-symbols
+    '("nicefrac" [ "Font changing command" ] "Numerator" "Denominator"))
+   ;; enable fontifying
+   (when (and (featurep 'font-latex)
+	      (eq TeX-install-font-lock 'font-latex-setup))
+     (add-to-list 'font-latex-match-textual-keywords-local "nicefrac")
+     (font-latex-match-textual-make))))
+
+(defvar LaTeX-nicefrac-package-options '("nice" "ugly")
+  "Package options for the nicefrac package.")
 
 ;;; nicefrac.el ends here
